package net.wanji.opt.service.impl;

import lombok.extern.slf4j.Slf4j;
import net.wanji.opt.common.Constants;
import net.wanji.opt.common.enums.CrossStatusEnum;
import net.wanji.opt.dto.*;
import net.wanji.opt.po.CrossDataRealtimePO;
import net.wanji.opt.service.CrossOptimizeService;
import net.wanji.opt.service.CrossPhaseService;
import net.wanji.opt.service.CrossSchedulesService;
import net.wanji.opt.service.CrossSchemeService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * @author hfx
 * @date 2023/1/10 15:15
 * @desc CrossOptimizeServiceImpl
 */
@Service
@Slf4j
public class CrossOptimizeServiceImpl implements CrossOptimizeService {

    @Resource
    CrossPhaseService crossPhaseService;

    @Resource
    CrossSchemeService crossSchemeService;

    @Resource
    CrossSchedulesService crossSchedulesService;


    public String realtimeOptimize() {

        // 获取路口实时数据
        List<CrossDataRealtimePO> crossDataRealtimePOList = listCrossDataRealtime();
        if(crossDataRealtimePOList.isEmpty()) {
            return null;
        }


        // 获取路口转向实时数据
        List<CrossTurnDataRealtimeDTO> turnDataRealtimeDTOList = listTurnDataRealtime();
        // 获取方案详情信息
        List<CrossSchemeDTO> schemeDTOList = crossSchemeService.listCrossSchemeDetails(null);
        // 获取调度详情信息
//        List<CrossSchedulesDTO> schedulesDTOList = crossSchedulesService.listCrossSchedulesDetails(null);



        // 获取异常路口数据
        List<CrossDataRealtimeDTO> abnormalCrossList = listAbnormalCross(crossDataRealtimePOList);
        if(abnormalCrossList.isEmpty()) {
            return null;
        }
        log.info("加载异常路口：{}条", abnormalCrossList.size());
        for(CrossDataRealtimeDTO dto : abnormalCrossList) {



        }

        // 获取相位方案信息

        // 计算相位优化配时

        // 方案下发

        return null;
    }


    /**
     * 获取异常路口数据
     * @return
     */
    public List<CrossDataRealtimeDTO> listAbnormalCross(List<CrossDataRealtimePO> crossDataRealtimePOList) {

        List<CrossDataRealtimeDTO> dtoList = new ArrayList<>();
        CrossDataRealtimeDTO dto = null;
        int status = 0;
        for (CrossDataRealtimePO po: crossDataRealtimePOList) {

            // 获取失衡、拥堵、溢出路口数据
            if(Constants.NO.equals(po.getIsSpillover()) || Constants.NO.equals(po.getIsCongestion()) || Constants.NO.equals(po.getIsUnbalance())) {
                continue;
            }

            if(Constants.YES.equals(po.getIsSpillover())) {         // 溢出
                status = CrossStatusEnum.SPILLOVER.getCode();
            } else if(Constants.YES.equals(po.getIsCongestion())) { // 拥堵
                status = CrossStatusEnum.CONGESTION.getCode();
            } else if(Constants.YES.equals(po.getIsUnbalance())) {  // 失衡
                status = CrossStatusEnum.UNBALANCE.getCode();
            }
            dto = new CrossDataRealtimeDTO();
            dto.setCrossId(po.getCrossId());
            dto.setStatus(status);
            dto.setDirList(null);
            dto.setTurnList(null);
            dtoList.add(dto);
        }
        return dtoList;
    }

    /**
     * 获取转向实时数据
     * @return
     */
    public List<CrossTurnDataRealtimeDTO> listTurnDataRealtime() {

        return new ArrayList<>();
    }

    /**
     * 获取路口实时数据
     * @return
     */
    public List<CrossDataRealtimePO> listCrossDataRealtime() {

        return new ArrayList<>();
    }

}
